@@ -4,7 +4,7 @@ use warnings;
 use Carp;
 use Data::Dumper;
 use Scalar::Util 'looks_like_number';
-our $VERSION = '1.0.2';
+our $VERSION = '2.1.0';
 
 my $GlobalRef = {};
 my $THIS;
@@ -27,6 +27,9 @@ our @EXPORT = qw (
     DUK_TYPE_POINTER
     DUK_TYPE_LIGHTFUNC
     DUK_ENUM_INCLUDE_NONENUMERABLE
+    DUK_ENUM_INCLUDE_HIDDEN
+    DUK_ENUM_INCLUDE_SYMBOLS
+    DUK_ENUM_EXCLUDE_STRINGS
     DUK_ENUM_INCLUDE_INTERNAL
     DUK_ENUM_OWN_PROPERTIES_ONLY
     DUK_ENUM_ARRAY_INDICES_ONLY
@@ -95,11 +98,13 @@ use constant {
 
     # Enumeration flags for duk_enum()
     DUK_ENUM_INCLUDE_NONENUMERABLE => (1 << 0),
-    DUK_ENUM_INCLUDE_INTERNAL      => (1 << 1),
-    DUK_ENUM_OWN_PROPERTIES_ONLY   => (1 << 2),
-    DUK_ENUM_ARRAY_INDICES_ONLY    => (1 << 3),
-    DUK_ENUM_SORT_ARRAY_INDICES    => (1 << 4),
-    DUK_ENUM_NO_PROXY_BEHAVIOR     => (1 << 5),
+    DUK_ENUM_INCLUDE_HIDDEN        => (1 << 1),
+    DUK_ENUM_INCLUDE_SYMBOLS       => (1 << 2),
+    DUK_ENUM_EXCLUDE_STRINGS       => (1 << 3),
+    DUK_ENUM_OWN_PROPERTIES_ONLY   => (1 << 4),
+    DUK_ENUM_ARRAY_INDICES_ONLY    => (1 << 5),
+    DUK_ENUM_SORT_ARRAY_INDICES    => (1 << 6),
+    DUK_ENUM_NO_PROXY_BEHAVIOR     => (1 << 7),
 
     DUK_COMPILE_EVAL               => (1 << 3),
     DUK_COMPILE_FUNCTION           => (1 << 4),
@@ -126,7 +131,7 @@ use constant {
 sub new {
     my $class= shift;
     my $self = bless {}, $class;
-    my $duk = $self->{duk} = JavaScript::Duktape::Vm->new();
+    my $duk = $self->{duk} = JavaScript::Duktape::Vm->perl_duk_new();
     $self->{pid} = $$;
 
     # Initialize global stash 'PerlGlobalStash'
@@ -269,8 +274,6 @@ sub destroy {
     my $duk  = delete $self->{duk};
     local $@; #duk_desatroy_heap mess with $@!!
     return if !$duk;
-    delete $self->{duk};
-    $duk->gc(0);
     $duk->destroy_heap();
 }
 
@@ -294,28 +297,30 @@ use Carp;
 my $Duklib;
 
 BEGIN {
-    my $FunctionsMap = JavaScript::Duktape::C::libPath::getPath("FunctionsMap.pl");
+    my $FunctionsMap = _get_path("FunctionsMap.pl");
     require $FunctionsMap;
 
+    sub _get_path { &JavaScript::Duktape::C::libPath::getPath }
+
     $Duklib = $^O eq 'MSWin32' ?
-                JavaScript::Duktape::C::libPath::getPath('duktape.dll') :
-                JavaScript::Duktape::C::libPath::getPath('duktape.so');
+                _get_path('duktape.dll') :
+                _get_path('duktape.so');
 };
 
 use Inline C => config =>
-    typemaps => JavaScript::Duktape::C::libPath::getPath('typemap'),
-    INC => '-I' . JavaScript::Duktape::C::libPath::getPath('../C');
+    typemaps => _get_path('typemap'),
+    INC      => '-I' . _get_path('../C') . ' -I' .  _get_path('../C/lib');
     # myextlib => $Duklib,
-    # LIBS => '-L'. JavaScript::Duktape::C::libPath::getPath('../C') . ' -lduktape';
+    # LIBS     => '-L'. JavaScript::Duktape::C::libPath::getPath('../C') . ' -lduktape';
 
-use Inline C => JavaScript::Duktape::C::libPath::getPath('duktape_wrap.c');
-use Carp;
+use Inline C => _get_path('duk_perl.c');
 
 use Inline C => q{
     void poke_buffer(IV to, IV from, IV sz) {
         memcpy( to, from, sz );
     }
 };
+
 my $ptr_format = do {
     my $ptr_size = $Config{ptrsize};
     $ptr_size == 4 ?
@@ -323,6 +328,7 @@ my $ptr_format = do {
         "Q" :
         die("Unrecognized pointer size");
 };
+
 sub peek { unpack 'P'.$_[1], pack $ptr_format, $_[0] }
 sub pv_address { unpack($ptr_format, pack("p", $_[0])) }
 
@@ -393,6 +399,7 @@ sub push_perl {
     }
 }
 
+
 sub to_perl_object {
     my $self = shift;
     my $index = shift;
@@ -404,6 +411,7 @@ sub to_perl_object {
     });
 }
 
+
 sub to_perl {
     my $self = shift;
     my $index = shift;
@@ -507,6 +515,7 @@ sub to_perl {
     return $ret;
 }
 
+
 ##############################################
 # push functions
 ##############################################
@@ -541,6 +550,7 @@ sub push_function {
     }, $nargs);
 }
 
+
 sub push_c_function {
     my $self  = shift;
     my $sub   = shift;
@@ -583,6 +593,7 @@ sub push_c_function {
     $self->set_finalizer(-2);
 }
 
+
 sub cache {
     my $self = shift;
     my $sub = shift;
@@ -601,6 +612,7 @@ sub cache {
     return $GlobalRef->{$code_cache_name};
 }
 
+
 #####################################################################
 # safe call
 #####################################################################
@@ -624,15 +636,19 @@ sub safe_call {
             if ($newtop > $oldtop){
                 $self->pop_n($newtop - $oldtop);
             }
-            $error =~ s/\n//g;
-            $error =~ s/\\/\\\\/g;
-            $self->eval_string("var t = new Error('$error'); t;");
+
+            $self->eval_string('(function (e){ return new Error(e) })');
+            $self->push_string($error);
+            $self->call(1);
         }
         return 1;
     }
+
+    #safe_call success returns 0
     return 0;
 }
 
+
 ##############################################
 # custom functions
 ##############################################
@@ -640,20 +656,23 @@ sub safe_call {
 *push_perl_function = \&push_c_function;
 *push_light_function = \&perl_push_function;
 
+
 ##############################################
 # overridden functions
 ##############################################
 *require_context = \&perl_duk_require_context;
 
+
 ##############################################
 # helper functions
 ##############################################
 *reset_top = \&perl_duk_reset_top;
 
+
 sub dump {
     my $self = shift;
     my $name = shift || "Duktape";
-    my $fh  = shift || \*STDOUT;
+    my $fh   = shift || \*STDOUT;
     my $n = $self->get_top();
     printf $fh "%s (top=%ld):", $name, $n;
     for (my $i = 0; $i < $n; $i++) {
@@ -665,8 +684,10 @@ sub dump {
     printf $fh "\n";
 }
 
+
 sub DESTROY {}
 
+
 package JavaScript::Duktape::Bool; {
     use warnings;
     use strict;
